@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import re
 from typing import NamedTuple, Optional, Tuple, Union
@@ -282,7 +277,7 @@ def broadcastable_shapes(
 _DIMENSION = r"\w+\??"  # Note that \w permits digits too!
 _SHAPE = r"\((?:{0}(?:,{0})".format(_DIMENSION) + r"{0,31})?\)"
 _ARGUMENT_LIST = "{0}(?:,{0})*".format(_SHAPE)
-_SIGNATURE = fr"^{_ARGUMENT_LIST}->{_SHAPE}$"
+_SIGNATURE = rf"^{_ARGUMENT_LIST}->{_SHAPE}$"
 _SIGNATURE_MULTIPLE_OUTPUT = r"^{0}->{0}$".format(_ARGUMENT_LIST)
 
 
@@ -299,7 +294,7 @@ def _hypothesis_parse_gufunc_signature(signature, all_checks=True):
                 "Hypothesis does not yet support generalised ufunc signatures "
                 "with multiple output arrays - mostly because we don't know of "
                 "anyone who uses them!  Please get in touch with us to fix that."
-                f"\n (signature={signature!r})"
+                f"\n ({signature=})"
             )
         if re.match(
             (
@@ -310,7 +305,7 @@ def _hypothesis_parse_gufunc_signature(signature, all_checks=True):
             signature,
         ):
             raise InvalidArgument(
-                f"signature={signature!r} matches Numpy's regex for gufunc signatures, "
+                f"{signature=} matches Numpy's regex for gufunc signatures, "
                 f"but contains shapes with more than {NDIM_MAX} dimensions and is thus invalid."
             )
         raise InvalidArgument(f"{signature!r} is not a valid gufunc signature")
@@ -369,7 +364,6 @@ def mutually_broadcastable_shapes(
     * ``base_shape`` is the shape against which all generated shapes can broadcast.
       The default shape is empty, which corresponds to a scalar and thus does
       not constrain broadcasting at all.
-    * ``shape`` is a tuple of integers.
     * ``min_dims`` is the smallest length that the generated shape can possess.
     * ``max_dims`` is the largest length that the generated shape can possess,
       defaulting to ``max(len(shape), min_dims) + 2``.
@@ -393,8 +387,8 @@ def mutually_broadcastable_shapes(
         ...     print(strat.example())
         BroadcastableShapes(input_shapes=((4, 1, 3), (4, 2, 3), ()), result_shape=(4, 2, 3))
         BroadcastableShapes(input_shapes=((3,), (1, 3), (2, 3)), result_shape=(2, 3))
-        BroadcastableShapes(input_shapes=((), (), ()), result_shape=(2, 3))
-        BroadcastableShapes(input_shapes=((3,), (), (3,)), result_shape=(2, 3))
+        BroadcastableShapes(input_shapes=((), (), ()), result_shape=())
+        BroadcastableShapes(input_shapes=((3,), (), (3,)), result_shape=(3,))
         BroadcastableShapes(input_shapes=((1, 2, 3), (3,), ()), result_shape=(1, 2, 3))
 
     """
@@ -590,13 +584,14 @@ class MutuallyBroadcastableShapesStrategy(st.SearchStrategy):
                 # only a singleton is valid in alignment with the base-dim
                 dim_side = 1
 
+            allowed_sides = sorted([1, dim_side])  # shrink to 0 when available
             for shape_id, shape in enumerate(shapes):
                 # Populating this dimension-size for each shape, either
                 # the drawn size is used or, if permitted, a singleton
                 # dimension.
-                if dim_count <= len(base_shape) and self.size_one_allowed:
+                if dim <= len(result_shape) and self.size_one_allowed:
                     # aligned: shrink towards size 1
-                    side = data.draw(st.sampled_from([1, dim_side]))
+                    side = data.draw(st.sampled_from(allowed_sides))
                 else:
                     side = dim_side
 
@@ -632,12 +627,24 @@ class MutuallyBroadcastableShapesStrategy(st.SearchStrategy):
 
 
 class BasicIndexStrategy(st.SearchStrategy):
-    def __init__(self, shape, min_dims, max_dims, allow_ellipsis, allow_newaxis):
+    def __init__(
+        self,
+        shape,
+        min_dims,
+        max_dims,
+        allow_ellipsis,
+        allow_newaxis,
+        allow_fewer_indices_than_dims,
+    ):
         self.shape = shape
         self.min_dims = min_dims
         self.max_dims = max_dims
         self.allow_ellipsis = allow_ellipsis
         self.allow_newaxis = allow_newaxis
+        # allow_fewer_indices_than_dims=False will disable generating indices
+        # that don't cover all axes, i.e. indices that will flat index arrays.
+        # This is necessary for the Array API as such indices are not supported.
+        self.allow_fewer_indices_than_dims = allow_fewer_indices_than_dims
 
     def do_draw(self, data):
         # General plan: determine the actual selection up front with a straightforward
@@ -673,7 +680,7 @@ class BasicIndexStrategy(st.SearchStrategy):
             while j < len(result) and result[j] == slice(None):
                 j += 1
             result[i:j] = [Ellipsis]
-        else:
+        elif self.allow_fewer_indices_than_dims:  # pragma: no cover
             while result[-1:] == [slice(None, None)] and data.draw(st.integers(0, 7)):
                 result.pop()
         if len(result) == 1 and data.draw(st.booleans()):
